package org.elsysbg.courses.ip.jsf.simpleForum.model;

import java.util.ArrayList;
import java.util.List;

import org.elsysbg.courses.ip.jsf.simpleForum.helper.CommonHelper;

public class ForumData {

	final private List<ForumPost> posts;
	final private List<ForumUser> users;
	
	private int nextPostId;
	private int nextUserId;
	static private ForumData instance;
	
	public ForumData() {
		if(instance!=null) {
			throw new IllegalStateException("ForumData must be singleton (no two instances may exists)");
		}
		instance = this;
		
		posts = new ArrayList<ForumPost>();
		users = new ArrayList<ForumUser>();
		nextPostId = 1;
		nextUserId = 1;
		
		final ForumUser administrator = new ForumUser();
		administrator.setUsername("admin");
		administrator.setPassword("admin");
		
		addUser(administrator);
	}
	
	static public ForumData getInstance() {
		if (instance == null) {
			new ForumData();
			//instance = this; in constructor
			CommonHelper.getApplicationMap().put("forumData", instance);
		}
		return instance;
	}
	
	synchronized public List<ForumPost> getPosts() {
		//prevent synchronization problems
		return new ArrayList<ForumPost>(posts);
	}
	
	synchronized public void addPost(ForumPost post) {
		//choose unique id
		post.setId(nextPostId++);
		posts.add(post);
	}
	
	synchronized public List<ForumUser> getUsers() {
		//prevent synchronization problems
		return new ArrayList<ForumUser>(users);
	}
	
	synchronized public boolean addUser(ForumUser user) {
		//check if user with this username already exist
		for(ForumUser existingUser: users) {
			if(existingUser.getUsername().equalsIgnoreCase(user.getUsername())) {
				return false;
			}
		}

		//choose unique id
		user.setId(nextUserId++);
		users.add(user);	
		return true;
	}
}
